using Microsoft.Office.InfoPath;
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Xml;
using System.Xml.XPath;

namespace MoiEventScheduler
{
    public partial class FormCode
    {
        private const string UserIsAuthorizedStr = "UserIsAuthorized";

        public void InternalStartup()
        {
            EventManager.FormEvents.Loading += new LoadingEventHandler(FormEvents_Loading);
            EventManager.XmlEvents["/my:myFields"].Changing += new XmlChangingEventHandler(myFields_Changing);
        }

        /// <summary>
        /// Remembers if this user is authorized.
        /// </summary>
        private bool UserIsAuthorized
        {
            get
            {
                return (bool)FormState[UserIsAuthorizedStr];
            }
            set
            {
                FormState[UserIsAuthorizedStr] = value;
            }
        }

        public void myFields_Changing(object sender, XmlChangingEventArgs e)
        {
            // Once a user is validated we won't prompt again
            if (UserIsAuthorized)
                return;

            string password = new MyPasswordDialog().Prompt();

            // Note: Including passwords as clear text in code is poor security
            if (password == "InfoPath!")
            {
                // Allow access
                e.CancelableArgs.Cancel = false;
                // Remember they have access
                UserIsAuthorized = true;
                MessageBox.Show("Thank you. Access has been granted for this session.", "Access Granted");
            }
            else
            {
                // Deny access with a dialog
                e.CancelableArgs.Cancel = true;
                e.CancelableArgs.Message = "Authorized Users Only";
                e.CancelableArgs.MessageDetails = "Only authorized users can schedule morale events.";
            }
        }

public void FormEvents_Loading(object sender, LoadingEventArgs e)
{
    FormState.Add(UserIsAuthorizedStr, false);
}
    }

    /// <summary>
    /// My own dialog for prompting a user for a password.
    /// </summary>
    public class MyPasswordDialog : Form
    {
        TextBox pwdTextbox;
        Button okButton, cancelButton;
        FlowLayoutPanel flow;
        DialogResult result;

        public MyPasswordDialog()
        {
            // Password text box
            pwdTextbox = new TextBox();
            pwdTextbox.MaxLength = 15;
            pwdTextbox.UseSystemPasswordChar = true;
            pwdTextbox.Size = new Size(150, pwdTextbox.Size.Width);
            // Catch Enter and Esc keys for convenience
            pwdTextbox.KeyDown += new KeyEventHandler(pwdTextbox_KeyDown);

            // OK button
            okButton = new Button();
            okButton.Text = "OK";
            okButton.Click += new EventHandler(okButton_Click);

            // Cancel button
            cancelButton = new Button();
            cancelButton.Text = "Cancel";
            cancelButton.Click += new EventHandler(cancelButton_Click);

            // Flow layout
            flow = new FlowLayoutPanel();
            flow.Dock = DockStyle.Fill;
            flow.Location = new Point(4, 4);
            flow.Size = new Size(100, 300);
            flow.TabIndex = 0;

            // This form
            Text = "Enter a password to continue";
            AutoSize = true;
            ClientSize = new Size(185, 65);
            Controls.Add(flow);
            FormBorderStyle = FormBorderStyle.FixedToolWindow;
            MaximizeBox = false;
            MinimizeBox = false;

            // Default for result
            result = DialogResult.Cancel;
        }

        // Handles keys pressed in the password dialog
        void pwdTextbox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape)
                cancelButton_Click(null, null);
            else if (e.KeyCode == Keys.Enter)
                okButton_Click(null, null);
        }

        // Handles Cancel button click
        void cancelButton_Click(object sender, EventArgs e)
        {
            result = DialogResult.Cancel;
            Close();
        }

        // Handles OK button click
        void okButton_Click(object sender, EventArgs e)
        {
            result = DialogResult.OK;
            Close();
        }

        // Show prompt and return password
        public string Prompt()
        {
            flow.Controls.Add(pwdTextbox);
            flow.Controls.Add(okButton);
            flow.Controls.Add(cancelButton);
            flow.ResumeLayout(false);
            flow.PerformLayout();
            PerformLayout();
            ShowDialog();

            if (result == DialogResult.OK)
                return pwdTextbox.Text;
            return string.Empty;
        }
    }
}
            
