using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Office.InfoPath.Server.Administration;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace FormsServicesTool
{
    public partial class MyProgram : Form
    {
        FormsService formsService = null;

        public MyProgram()
        {
            InitializeComponent();
        }

        private void UploadFormTemplate_Load(object sender, EventArgs e)
        {
            formsService = SPFarm.Local.Services.GetValue<FormsService>(FormsService.ServiceName);
        }

        private void BrowseButton_Click(object sender, EventArgs e)
        {
            openFileDialog1.FileName = string.Empty;
            openFileDialog1.Filter = "Form Template (.xsn) file|*.xsn";
            DialogResult result = openFileDialog1.ShowDialog();
            if (result == DialogResult.OK)
            {
                UploadLocation.Text = openFileDialog1.FileName;
            }
        }

        private void VerifyButton_Click(object sender, EventArgs e)
        {
            ConverterMessageCollection messages = FormTemplateCollection.VerifyFormTemplate(UploadLocation.Text);
            OperationOutput.Text = String.Empty;
            foreach (ConverterMessage message in messages)
            {
                OperationOutput.Text += message.ShortMessage + Environment.NewLine 
                    + message.DetailedMessage + Environment.NewLine + Environment.NewLine;
            }
            if (string.IsNullOrEmpty(OperationOutput.Text))
            {
                OperationOutput.Text = "No messages to report from the server.";
            }
        }

        private void UploadButton_Click(object sender, EventArgs e)
        {
            OperationOutput.Text = "Uploading form template";
            FormTemplateCollection collection = formsService.FormTemplates;
            try
            {
                collection.UploadFormTemplate(UploadLocation.Text);
            }
            catch (Exception ex)
            {
                OperationOutput.Text = "There was a problem uploading to the server. " + ex.Message;
                return;
            }

            // wait for up to 2 minutes for the form template to upload across the farm
            for (int i = 0; i < 120; i++)
            {
                OperationOutput.Text += ".";
                System.Threading.Thread.Sleep(1000);

                if (collection.ItemFromFile(UploadLocation.Text).FormTemplateStatus == FormTemplate.FormTemplateState.Normal)
                {
                    OperationOutput.Text += " upload complete" + Environment.NewLine;
                    break;
                }
            }

            OperationOutput.Text += "Activating form template..." + Environment.NewLine;

            collection.ItemFromFile(UploadLocation.Text).Activate(new SPSite("http://localhost"));
            OperationOutput.Text += "Done!" + Environment.NewLine;
        }

        private void ListButton_Click(object sender, EventArgs e)
        {
            OperationOutput.Text = string.Empty;
            foreach (FormTemplate formTemplate in formsService.FormTemplates)
            {
                OperationOutput.Text += formTemplate.Name + Environment.NewLine;
            }
        }
    }
}