using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.Office.InfoPath.Server.Administration;
using Microsoft.Office.Server.Administration;
using Microsoft.SharePoint.Administration;

namespace Quiesce
{
    public partial class Form1 : Form
    {
        SessionStateService sessionStateService = null;
        FormsService formsService = null;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            sessionStateService = SPFarm.Local.Services.GetValue<SessionStateService>(String.Empty);
            formsService = SPFarm.Local.Services.GetValue<FormsService>(FormsService.ServiceName);
            foreach (FormTemplate template in formsService.FormTemplates)
            {
                formTemplates.Items.Add(template.DisplayName);
            }
        }

        private void formTemplates_SelectedIndexChanged(object sender, EventArgs e)
        {
            RefreshStatus();
        }

        private void quiesceButton_Click(object sender, EventArgs e)
        {
            if (!IsValidTime())
            {
                toolStripStatusLabel2.Text = "Please enter a valid time in minutes.";
                return;
            }
            if (SelectedFormTemplate != null)
            {
                SelectedFormTemplate.Quiesce(DateTimeSpan);
            }
            RefreshStatus();
        }

        private TimeSpan DateTimeSpan
        {
            get
            {
                return new TimeSpan(0, int.Parse(timeUntilQuiesced.Text), 0);
            }
        }

        private FormTemplate SelectedFormTemplate
        {
            get
            {
                object selectedTemplate = formTemplates.SelectedItem;
                if (selectedTemplate != null)
                {
                    return formsService.FormTemplates[selectedTemplate.ToString()];
                }
                return null;
            }
        }

        private bool IsValidTime()
        {
            int quiesceTime;
            return int.TryParse(timeUntilQuiesced.Text, out quiesceTime);
        }

        private void timeUntilQuiesced_TextChanged(object sender, EventArgs e)
        {
            if (!IsValidTime())
            {
                timeUntilQuiesced.Text = "5";
            }
        }

        private void refreshStatus_Click(object sender, EventArgs e)
        {
            RefreshStatus();
        }

        private void RefreshStatus()
        {
            if (SelectedFormTemplate != null)
            {
                toolStripStatusLabel2.Text = "This template quiesce status: "
                    + SelectedFormTemplate.QuiesceStatus.ToString();
            }
        }

        private void unquiesceButton_Click(object sender, EventArgs e)
        {
            if (SelectedFormTemplate != null)
            {
                SelectedFormTemplate.Unquiesce();
            }
            RefreshStatus();
        }

        private void quiesceFarm_Click(object sender, EventArgs e)
        {
            if (!IsValidTime())
            {
                toolStripStatusLabel2.Text = "Please enter a valid time in minutes.";
                return;
            }
            sessionStateService.Quiesce(DateTimeSpan);
            RefreshFarmStatus();
        }

        private void RefreshFarmStatus()
        {
            toolStripStatusLabel2.Text = "The farm is " + sessionStateService.QuiesceState.ToString();
        }

        private void unquiesceFarm_Click(object sender, EventArgs e)
        {
            sessionStateService.Unquiesce();
            RefreshFarmStatus();
        }

        private void refreshFarmStatusButton_Click(object sender, EventArgs e)
        {
            RefreshFarmStatus();
        }
    }
}