// InfoPathChartControl.h : Declaration of the InfoPathChartControl
#pragma once
#include "resource.h"       // main symbols
#include <atlctl.h>
#include <gdiplus.h>

// This assumes that ipeditor.dll is installed at this location.
#import "C:\Program Files\Microsoft Office\Office12\ipeditor.dll" raw_interfaces_only no_namespace

using namespace Gdiplus;

// IInfoPathChartControl
[
	object,
	uuid(449938E5-49D2-4036-9C09-9BAF77C98BFE),
	dual,
	helpstring("IInfoPathChartControl Interface"),
	pointer_default(unique)
]
__interface IInfoPathChartControl : public IDispatch
{
    [propget, id(1), bindable, requestedit]
    HRESULT Value([out, retval] IDispatch** ppdispValue);

    [propput, id(1), bindable, requestedit]
    HRESULT Value([in] IDispatch* pdispValue);
};


// _IInfoPathChartControlEvents
[
	uuid("26D9054D-F812-4DED-B28D-00D9449E15CB"),
	dispinterface,
	helpstring("_IInfoPathChartControlEvents Interface")
]
__interface _IInfoPathChartControlEvents
{
};

// InfoPathChartControl
[
	coclass,
	control,
	default(IInfoPathChartControl, _IInfoPathChartControlEvents),
	threading(apartment),
	vi_progid("InfoPathChart.InfoPathChartControl"),
	progid("InfoPathChart.InfoPathChartControl.1"),
	version(1.0),
	uuid("386632B7-E11D-4DFA-8C38-8ACBFE870C03"),
	helpstring("InfoPath Chart Control Sample"),
	event_source(com),
	support_error_info(IInfoPathChartControl),
	registration_script("control.rgs")
]
class ATL_NO_VTABLE InfoPathChartControl :
	public CStockPropImpl<InfoPathChartControl, IInfoPathChartControl>,
	public IPersistStreamInitImpl<InfoPathChartControl>,
	public IOleControlImpl<InfoPathChartControl>,
	public IOleObjectImpl<InfoPathChartControl>,
	public IOleInPlaceActiveObjectImpl<InfoPathChartControl>,
	public IViewObjectExImpl<InfoPathChartControl>,
	public IOleInPlaceObjectWindowlessImpl<InfoPathChartControl>,
	public IPersistStorageImpl<InfoPathChartControl>,
	public ISpecifyPropertyPagesImpl<InfoPathChartControl>,
	public IQuickActivateImpl<InfoPathChartControl>,
	public IDataObjectImpl<InfoPathChartControl>,
	public IObjectSafetyImpl<InfoPathChartControl, INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA>,
	public CComControl<InfoPathChartControl>,
    public InfoPathControl
{
public:
	InfoPathChartControl()
	{
        // Initialize GDI+
        GdiplusStartupInput gdiplusStartupInput;

        GdiplusStartup(&m_gdiplusToken, &gdiplusStartupInput, NULL);
	}

    ~InfoPathChartControl()
    {
        GdiplusShutdown(m_gdiplusToken);
    }

    DECLARE_OLEMISC_STATUS(OLEMISC_RECOMPOSEONRESIZE |
	    OLEMISC_CANTLINKINSIDE |
	    OLEMISC_INSIDEOUT |
	    OLEMISC_ACTIVATEWHENVISIBLE |
	    OLEMISC_SETCLIENTSITEFIRST)

    BEGIN_PROP_MAP(InfoPathChartControl)
	    PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	    PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	    // Example entries
	    // PROP_ENTRY("Property Description", dispid, clsid)
	    // PROP_PAGE(CLSID_StockColorPage)
    END_PROP_MAP()

    BEGIN_MSG_MAP(InfoPathChartControl)
	    CHAIN_MSG_MAP(CComControl<InfoPathChartControl>)
	    DEFAULT_REFLECTION_HANDLER()
    END_MSG_MAP()

    // Handler prototypes:
    //  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
    //  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    //  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

	__event __interface _IInfoPathChartControlEvents;

    // IViewObjectEx
    DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

    // IInfoPathChartControl
	HRESULT OnDraw(ATL_DRAWINFO& di)
	{
		RECT& rc = *(RECT*)di.prcBounds;

		// Set Clip region to the rectangle specified by di.prcBounds
		HRGN hRgnOld = NULL;
        static const Color chartColors[7] = 
        {
            Color::Red,
            Color::Green,
            Color::Blue,
            Color::Yellow,
            Color::AliceBlue,
            Color::Pink,
            Color::Purple,
        };

		Graphics graphics(di.hdcDraw); 

        graphics.SetSmoothingMode(SmoothingModeHighQuality);

        REAL startAngle = 0.0f;

        if (m_bInPlaceActive)
        {
            Rect ellipseRect(rc.left, rc.top, rc.right - rc.left - 1, rc.bottom - rc.top - 1);
            long cItems = m_rgSweepAngles.GetSize();

            for (long i = 0; i < cItems && i < ARRAYSIZE(chartColors); i++)
            {
                REAL sweepAngle = m_rgSweepAngles[i];
                SolidBrush chartBrush(chartColors[i]);

                graphics.FillPie(&chartBrush, ellipseRect, startAngle, sweepAngle);
                startAngle += sweepAngle;
            }
        }
        else
        {
            Rect ellipseRect(rc.left, rc.top, rc.right - rc.left - 1, rc.bottom - rc.top - 1);

            // If in InfoPath design mode m_bInPlaceActive will be FALSE. 
            // In that case just show the standard sweep angle of 120 degrees.
            // If we are in "fill out a form" mode, then use the values 
            // from the form data.
            REAL sweepAngle = 120.0f;
            
            for (int i = 0; i < 3; i++)
            {
                SolidBrush chartBrush(chartColors[i]);

                graphics.FillPie(&chartBrush, ellipseRect, startAngle, sweepAngle);
                startAngle += sweepAngle;
            }
        }

        return S_OK;
	}

    // InfoPathControl
    STDMETHOD(Init)(InfoPathControlSite* pControlSite);
    STDMETHOD(Uninit)();
    STDMETHOD(SaveState)();
    STDMETHOD(Enable)(VARIANT_BOOL vfEnabled);

	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

	void FinalRelease()
	{
	}

    STDMETHOD(get_Value)(IDispatch** ppdispValue);
    STDMETHOD(put_Value)(IDispatch* pdispValue);

private:
    CSimpleArray<REAL> m_rgSweepAngles;
    ULONG_PTR m_gdiplusToken;
    CComPtr<InfoPathControlSite> m_spControlSite;

};
