// SimpleInfoPathControl.h : Declaration of the SimpleInfoPathControl
#pragma once
#include "resource.h"       // main symbols
#include <atlctl.h>
#include "richedit.h"

#define DISPID_EDIT_VALUE 1

// This assumes that ipeditor.dll is installed at this location.
#import "C:\Program Files\Microsoft Office\Office12\ipeditor.dll" raw_interfaces_only no_namespace

// ISimpleInfoPathControl
[
	object,
	uuid(38F626EC-7806-40BC-9063-1D05CBFBF94C),
	dual,
	helpstring("ISimpleInfoPathControl Interface"),
	pointer_default(unique)
]
__interface ISimpleInfoPathControl : public IDispatch
{
	[propput, bindable, requestedit, id(DISPID_BACKCOLOR)]
	HRESULT BackColor([in]OLE_COLOR clr);
	[propget, bindable, requestedit, id(DISPID_BACKCOLOR)]
	HRESULT BackColor([out,retval]OLE_COLOR* pclr);
	[propput, bindable, requestedit, id(DISPID_ENABLED)]
	HRESULT Enabled([in]VARIANT_BOOL vbool);
	[propget, bindable, requestedit, id(DISPID_ENABLED)]
	HRESULT Enabled([out,retval]VARIANT_BOOL* pbool);
    [propput, bindable, requestedit, id(DISPID_EDIT_VALUE)]
    HRESULT Value([in] BSTR newVal);
    [propget, bindable, requestedit, id(DISPID_EDIT_VALUE)]
    HRESULT Value([out, retval] BSTR* pVal);
};


// _ISimpleInfoPathControlEvents
[
	uuid("2C5F3159-7E12-48CF-8646-7588FE0D9DE6"),
	dispinterface,
	helpstring("_ISimpleInfoPathControlEvents Interface")
]
__interface _ISimpleInfoPathControlEvents
{
};

// SimpleInfoPathControl
[
	coclass,
	control,
	default(ISimpleInfoPathControl, _ISimpleInfoPathControlEvents),
	threading(apartment),
	vi_progid("SimpleInfoPathActiveX.SimpleInfoPathCon"),
	progid("SimpleInfoPathActiveX.SimpleInfoPathC.1"),
	version(1.0),
	uuid("3B8D2C10-4C71-4C24-83C0-C6628092B63F"),
	helpstring("Simple InfoPath Control Sample"),
	event_source(com),
	support_error_info(ISimpleInfoPathControl),
	registration_script("control.rgs")
]
class ATL_NO_VTABLE SimpleInfoPathControl :
	public CStockPropImpl<SimpleInfoPathControl, ISimpleInfoPathControl>,
	public IPersistStreamInitImpl<SimpleInfoPathControl>,
	public IOleControlImpl<SimpleInfoPathControl>,
	public IOleObjectImpl<SimpleInfoPathControl>,
	public IOleInPlaceActiveObjectImpl<SimpleInfoPathControl>,
	public IViewObjectExImpl<SimpleInfoPathControl>,
	public IOleInPlaceObjectWindowlessImpl<SimpleInfoPathControl>,
	public IPersistStorageImpl<SimpleInfoPathControl>,
	public ISpecifyPropertyPagesImpl<SimpleInfoPathControl>,
	public IQuickActivateImpl<SimpleInfoPathControl>,
	public IDataObjectImpl<SimpleInfoPathControl>,
    public IObjectSafetyImpl<SimpleInfoPathControl, INTERFACESAFE_FOR_UNTRUSTED_CALLER | INTERFACESAFE_FOR_UNTRUSTED_DATA>,
    public IPersistPropertyBagImpl<SimpleInfoPathControl>,
    public CComControl<SimpleInfoPathControl>,
    public InfoPathControl
{
public:
	CContainedWindow m_ctlRichEdit;

#pragma warning(push)
#pragma warning(disable: 4355) // 'this' : used in base member initializer list

	SimpleInfoPathControl()
		: m_ctlRichEdit(RICHEDIT_CLASS, this, 1)
	{
		m_bWindowOnly = TRUE;
	}

#pragma warning(pop)

    DECLARE_OLEMISC_STATUS(OLEMISC_RECOMPOSEONRESIZE |
	    OLEMISC_CANTLINKINSIDE |
	    OLEMISC_INSIDEOUT |
	    OLEMISC_ACTIVATEWHENVISIBLE |
	    OLEMISC_SETCLIENTSITEFIRST)

    BEGIN_PROP_MAP(SimpleInfoPathControl)
	    PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	    PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	    PROP_ENTRY("BackColor", DISPID_BACKCOLOR, CLSID_StockColorPage)
	    PROP_ENTRY("Enabled", DISPID_ENABLED, CLSID_NULL)
	    // Example entries
	    // PROP_ENTRY("Property Description", dispid, clsid)
	    // PROP_PAGE(CLSID_StockColorPage)
    END_PROP_MAP()

    BEGIN_MSG_MAP(SimpleInfoPathControl)
	    MESSAGE_HANDLER(WM_CREATE, OnCreate)
	    MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
        MESSAGE_HANDLER(WM_COMMAND, OnCommand)
        CHAIN_MSG_MAP(CComControl<SimpleInfoPathControl>)
    ALT_MSG_MAP(1)
	    // Replace this with message map entries for superclassed Edit
    END_MSG_MAP()

    // Handler prototypes:
    //  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
    //LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
    //LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);

    LRESULT OnCommand(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
    {
        switch(uMsg)
        {
            case WM_COMMAND:
                if (HIWORD(wParam) == EN_KILLFOCUS)
                {
                    // Alert InfoPath that the bound property has changed.
                    FireOnChanged(DISPID_EDIT_VALUE);
                }
            break;

            case WM_CTLCOLOREDIT:
                if (m_ctlRichEdit.IsWindow())
                {
                    ::SetBkColor((HDC)wParam, m_clrBackColor);
                }
                break;
        }

        bHandled = FALSE;
        return 1;
    }

	BOOL PreTranslateAccelerator(LPMSG pMsg, HRESULT& hRet)
	{
		if (pMsg->message == WM_KEYDOWN)
		{
			switch(pMsg->wParam)
			{
			case VK_LEFT:
			case VK_RIGHT:
			case VK_UP:
			case VK_DOWN:
			case VK_HOME:
			case VK_END:
			case VK_NEXT:
			case VK_PRIOR:
				hRet = S_FALSE;
				return TRUE;
			}
		}

		//TODO: Add your additional accelerator handling code here
		return FALSE;
	}

	LRESULT OnSetFocus(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
	{
		LRESULT lRes = CComControl<SimpleInfoPathControl>::OnSetFocus(uMsg, wParam, lParam, bHandled);
		if (m_bInPlaceActive && m_ctlRichEdit.IsWindow())
		{
			if (!IsChild(::GetFocus()))
            {
				m_ctlRichEdit.SetFocus();
            }
		}

		return lRes;
	}

	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		RECT rc;

		GetWindowRect(&rc);

		rc.right -= rc.left;
		rc.bottom -= rc.top;
		rc.top = rc.left = 0;

		m_ctlRichEdit.Create(m_hWnd, rc, NULL, 
            WS_BORDER | WS_CHILD | WS_VISIBLE | WS_VSCROLL | ES_MULTILINE | ES_WANTRETURN, 
            WS_EX_CLIENTEDGE);

        ::SendMessage((HWND)m_ctlRichEdit, EM_SETBKGNDCOLOR, 0, m_clrBackColor);

		return 0;
	}

	STDMETHOD(SetObjectRects)(LPCRECT prcPos, LPCRECT prcClip)
	{
		IOleInPlaceObjectWindowlessImpl<SimpleInfoPathControl>::SetObjectRects(prcPos, prcClip);
		int cx, cy;

		cx = prcPos->right - prcPos->left;
		cy = prcPos->bottom - prcPos->top;

		::SetWindowPos(m_ctlRichEdit.m_hWnd, NULL, 0, 0, cx, cy, SWP_NOZORDER | SWP_NOACTIVATE);
		return S_OK;
	}

	__event __interface _ISimpleInfoPathControlEvents;

    // CComControl overrides
	virtual HRESULT OnDraw(ATL_DRAWINFO& di)
	{
        if (m_bInPlaceActive)
        {
            // This control is only meant to run in InfoPath
            // Disable it if there is no InfoPathControlSite.
            if (m_spControlSite == NULL)
            {
                m_ctlRichEdit.SetWindowTextW(L"This control only works in InfoPath.");
                m_ctlRichEdit.EnableWindow(FALSE);
            }
        }
        else
		{
			HPEN hPen = (HPEN)::GetStockObject(BLACK_PEN);
			HBRUSH hBrush;

            if (m_clrBackColor == -1)
            {
			    hBrush = (HBRUSH)::GetStockObject(LTGRAY_BRUSH);
            }
            else
            {
                LOGBRUSH logBrush;

                logBrush.lbStyle = BS_SOLID;
                logBrush.lbColor = m_clrBackColor;

                hBrush = ::CreateBrushIndirect(&logBrush);
            }

			::SelectObject(di.hdcDraw, hPen);
			::SelectObject(di.hdcDraw, hBrush);

			::Rectangle(di.hdcDraw, di.prcBounds->left, di.prcBounds->top, di.prcBounds->right, di.prcBounds->bottom);

			::SetTextColor(di.hdcDraw, ::GetSysColor(COLOR_WINDOWTEXT));
			::SetBkMode(di.hdcDraw, TRANSPARENT);

			::DrawText(di.hdcDraw, _T("Simple InfoPath Control"), -1, (LPRECT)di.prcBounds, DT_CENTER | DT_SINGLELINE | DT_VCENTER);

            ::DeleteObject(hBrush);
		}

        return S_OK;
	}

    // IViewObjectEx
	DECLARE_VIEW_STATUS(VIEWSTATUS_SOLIDBKGND | VIEWSTATUS_OPAQUE)

    // ISimpleInfoPathControl
	STDMETHOD_(void, OnBackColorChanged)();
	STDMETHOD_(void, OnEnabledChanged)();
    STDMETHOD(get_Value)(BSTR* pVal);
    STDMETHOD(put_Value)(BSTR newVal);

    // InfoPathControl
    STDMETHOD(Init)(InfoPathControlSite* pControlSite);
    STDMETHOD(Uninit)();
    STDMETHOD(SaveState)();
    STDMETHOD(Enable)(VARIANT_BOOL vfEnabled);

	DECLARE_PROTECT_FINAL_CONSTRUCT()

	HRESULT FinalConstruct()
	{
		return S_OK;
	}

	void FinalRelease()
	{
	}

    BOOL m_bEnabled;
    OLE_COLOR m_clrBackColor;

private:
    CComPtr<InfoPathControlSite> m_spControlSite;
    CComBSTR m_bstrValue;
};
