namespace FormDataTaskPane
{
    using System;
    using Extensibility;
    using System.Runtime.InteropServices;
    using System.Reflection;
    using System.Windows.Forms;
    using System.Collections;
    using System.Collections.Specialized;
    using Microsoft.Office.Core;
    using Microsoft.Office.Interop.InfoPath;
    using FormDataTaskPaneActiveX;

    #region IFormDataViewAddIn

    [ComVisible(true)]
    [InterfaceType(ComInterfaceType.InterfaceIsIUnknown)]
    [Guid("3995A8BB-7B96-450a-AC72-E2C46F2315D4")]
    public interface IFormDataViewAddIn
    {
        void OnTaskPaneReady();
    }

    #endregion

	#region Read me for Add-in installation and setup information.
	// When run, the Add-in wizard prepared the registry for the Add-in.
	// At a later time, if the Add-in becomes unavailable for reasons such as:
	//   1) You moved this project to a computer other than which is was originally created on.
	//   2) You chose 'Yes' when presented with a message asking if you wish to remove the Add-in.
	//   3) Registry corruption.
	// you will need to re-register the Add-in by building the FormDataTaskPaneSetup project, 
	// right click the project in the Solution Explorer, then choose install.
	#endregion
	
	/// <summary>
	///   The object for implementing an Add-in.
	/// </summary>
	/// <seealso class='IDTExtensibility2' />
	[Guid("16DA8933-0CD3-461A-A028-E5A8B71C1E22"), ProgId("FormDataTaskPane.Connect")]
    public class FormDataViewAddIn : Object, Extensibility.IDTExtensibility2, ICustomTaskPaneConsumer, IFormDataViewAddIn
	{
		/// <summary>
		///		Implements the constructor for the Add-in object.
		///		Place your initialization code within this method.
		/// </summary>
		public FormDataViewAddIn()
		{
            _taskPanes = new ListDictionary();
        }

        /// <summary>
        ///      Implements the OnConnection method of the IDTExtensibility2 interface.
        ///      Receives notification that the Add-in is being loaded.
        /// </summary>
        /// <param term='application'>
        ///      Root object of the host application.
        /// </param>
        /// <param term='connectMode'>
        ///      Describes how the Add-in is being loaded.
        /// </param>
        /// <param term='addInInst'>
        ///      Object representing this Add-in.
        /// </param>
        /// <seealso class='IDTExtensibility2' />
        public void OnConnection(
                        object application,
                        Extensibility.ext_ConnectMode connectMode,
                        object addInInst,
                        ref System.Array custom)
        {
            try
            {
                _applicationObject = (ApplicationClass)application;
            }
            catch (Exception)
            {
                MessageBox.Show("The InfoPath Custom Task Pane Add-in is only supported in Microsoft Office InfoPath", DialogCaption);
                throw;
            }

            if (connectMode == ext_ConnectMode.ext_cm_Startup)
            {
                ((COMAddIn)addInInst).Object = this;

                // _applicationEvents has to be a data member in order
                // for the event sinks to remain attached throughout the 
                // lifetime of the application.
                _applicationEvents = (ApplicationEvents)_applicationObject.Events;

                _applicationEvents.XDocumentOpen += new _ApplicationEvents_XDocumentOpenEventHandler(_applicationEvents_XDocumentOpen);
                _applicationEvents.NewXDocument += new _ApplicationEvents_NewXDocumentEventHandler(_applicationEvents_NewXDocument);
                _applicationEvents.XDocumentBeforeClose += new _ApplicationEvents_XDocumentBeforeCloseEventHandler(_applicationEvents_XDocumentBeforeClose);
            }
        }

        /// <summary>
        ///     Implements the OnDisconnection method of the IDTExtensibility2 interface.
        ///     Receives notification that the Add-in is being unloaded.
        /// </summary>
        /// <param term='disconnectMode'>
        ///      Describes how the Add-in is being unloaded.
        /// </param>
        /// <param term='custom'>
        ///      Array of parameters that are host application specific.
        /// </param>
        /// <seealso class='IDTExtensibility2' />
        public void OnDisconnection(Extensibility.ext_DisconnectMode disconnectMode, ref System.Array custom)
        {
            _applicationObject = null;
            _applicationEvents = null;
            _ctpFactory = null;
            _taskPanes = null;
        }

        /// <summary>
        ///      Implements the OnAddInsUpdate method of the IDTExtensibility2 interface.
        ///      Receives notification that the collection of Add-ins has changed.
        /// </summary>
        /// <param term='custom'>
        ///      Array of parameters that are host application specific.
        /// </param>
        /// <seealso class='IDTExtensibility2' />
        public void OnAddInsUpdate(ref System.Array custom)
        {
        }

        /// <summary>
        ///      Implements the OnStartupComplete method of the IDTExtensibility2 interface.
        ///      Receives notification that the host application has completed loading.
        /// </summary>
        /// <param term='custom'>
        ///      Array of parameters that are host application specific.
        /// </param>
        /// <seealso class='IDTExtensibility2' />
        public void OnStartupComplete(ref System.Array custom)
        {
            // The application window is now ready. We can 
            // access the CommandBars collection of the window
            // and add our button.
            CommandBar toolbar = CreateToolbar();

            // Now, add our button if it's not already there.
            try
            {
                // See if the buttons already exist.
                CommandBarButton toggleButton = GetButton(ToggleButtonTag);
                CommandBarButton refreshButton = GetButton(RefreshButtonTag);

                // Add the buttons if needed. Add the refresh button first
                // so it will be after the toggle button on the toolbar.
                AddButton(ref refreshButton,
                          RefreshCaption,
                          "Refreshes the data in the Custom Task Pane",
                          RefreshButtonTag,
                          true /*beginGroup*/,
                          OnRefreshClick);

                AddButton(ref toggleButton,
                          ShowPaneCaption,
                          "Shows or hides the Custom Task Pane",
                          ToggleButtonTag,
                          false /*beginGroup*/,
                          OnToggleClick);

                toggleButton.Enabled = false;
                refreshButton.Enabled = false;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, DialogCaption);
            }
        }

        /// <summary>
        ///      Implements the OnBeginShutdown method of the IDTExtensibility2 interface.
        ///      Receives notification that the host application is being unloaded.
        /// </summary>
        /// <param term='custom'>
        ///      Array of parameters that are host application specific.
        /// </param>
        /// <seealso class='IDTExtensibility2' />
        public void OnBeginShutdown(ref System.Array custom)
        {
        }

        /// <summary>
        /// Called when an existing form is opened
        /// </summary>
        private void _applicationEvents_XDocumentOpen(_XDocument pDocument)
        {
            ((XDocument)pDocument).OnSwitchView += new _XDocumentEventSink2_OnSwitchViewEventHandler(FormDataViewAddIn_OnSwitchView);
        }

        /// <summary>
        /// Called when a new form is created from a form template
        /// </summary>
        private void _applicationEvents_NewXDocument(_XDocument pDocument)
        {
            ((XDocument)pDocument).OnSwitchView += new _XDocumentEventSink2_OnSwitchViewEventHandler(FormDataViewAddIn_OnSwitchView);
        }

        /// <summary>
        /// Called when switching to a new view or when the 
        /// form is initially loaded.
        /// Note that OnSwitchView may not fire if there is
        /// an event handler in business logic for the form.
        /// </summary>
        private void FormDataViewAddIn_OnSwitchView(DocEvent pEvent)
        {
            CommandBarButton toggleButton = GetButton(ToggleButtonTag);
            toggleButton.Enabled = true;
            toggleButton.Caption = ShowPaneCaption;

            CommandBarButton refreshButton = GetButton(RefreshButtonTag);
            refreshButton.Enabled = false;
        }

        /// <summary>
        /// Called when the document is about to close.
        /// </summary>
        private void _applicationEvents_XDocumentBeforeClose(_XDocument pDocument, ref bool pfCancel)
        {
            // When the document is closed due to the user closing the application, the
            // ActiveWindow object is released by the time this event handler is called.
            WindowObject window = _applicationObject.ActiveWindow;
            if (window != null)
            {
                int hashCode = window.GetHashCode();
                CustomTaskPane taskPane = (CustomTaskPane)_taskPanes[hashCode];

                if (taskPane != null) // The taskPane may not have been opened.
                {
                    taskPane.Visible = false;

                    IFormDataTPActiveXCtl ctpActiveXCtrl = (IFormDataTPActiveXCtl)taskPane.ContentControl;
                    ctpActiveXCtrl.Application = null;

                    taskPane.Delete();
                    taskPane = null;
                    _taskPanes[hashCode] = null;
                }

                // Disable the buttons for this window only.
                CommandBarButton toggleButton = GetButton(ToggleButtonTag);

                toggleButton.Enabled = false;
                toggleButton.Caption = ShowPaneCaption;

                CommandBarButton refreshButton = GetButton(RefreshButtonTag);
                refreshButton.Enabled = false;
            }
        }

        /// <summary>
        /// Creates the toolbar for our COM Add-in 
        /// if it doesn't already exist.
        /// </summary>
        private CommandBar CreateToolbar()
        {
            Window window = _applicationObject.ActiveWindow;
            if (window == null)
            {
                window = _applicationObject.Windows[0];
            }

            CommandBar toolbar = GetCurrentToolbar();
            if (toolbar == null)
            {
                CommandBars commandBars = (CommandBars)window.CommandBars;

                toolbar = commandBars.Add(
                                ToolbarName,
                                MsoBarPosition.msoBarTop,
                                false /*replace*/,
                                false /*temporary*/);
            }

            toolbar.Visible = true;
            return toolbar;
        }

        /// <summary>
        /// Returns the toolbar for the current window.
        /// </summary>
        private CommandBar GetCurrentToolbar()
        {
            return GetCommandBar(ToolbarName);
        }

        /// <summary>
        /// Returns the toolbar for the current window.
        /// </summary>
        private CommandBar GetCommandBar(string cmdBarName)
        {
            Window window = _applicationObject.ActiveWindow;
            if (window == null)
            {
                window = _applicationObject.Windows[0];
            }

            CommandBars commandBars = (CommandBars)window.CommandBars;
            CommandBar toolbar = null;

            // Referencing our command bar like this will throw an exception
            // if it doesn't exist. Alternatively, we could use the 
            // enumerator on the CommandBars object to look for our
            // command bar.
            try
            {
                toolbar = commandBars[cmdBarName];
            }
            catch (Exception)
            {
            }

            return toolbar;
        }

        /// <summary>
        ///     Adds a button to the command bar.
        /// </summary>
        /// <param term='tag'>
        ///      Tag of the button to find.
        /// </param>
        private void AddButton(ref CommandBarButton button,
                       string caption,
                       string tooltip,
                       string tag,
                       bool beginGroup,
                       _CommandBarButtonEvents_ClickEventHandler clickHandler)
        {
            if (button == null)
            {
                CommandBar toolbar = GetCurrentToolbar();

                button = (CommandBarButton)toolbar.Controls.Add(
                                MsoControlType.msoControlButton,
                                Missing.Value,
                                Missing.Value,
                                1,
                                false /*temporary*/);

                button.BeginGroup = beginGroup;
            }

            // Reset the button properties in case something failed
            // the last time we attempted to do so.
            button.Visible = true;
            button.Caption = caption;
            button.Style = MsoButtonStyle.msoButtonCaption;
            button.Tag = tag;
            button.TooltipText = tooltip;
            button.Click += new _CommandBarButtonEvents_ClickEventHandler(clickHandler);
        }

        /// <summary>
        ///     Retrieves a button from the commandbar if the button with 
        ///     the given tag ID exists.
        /// </summary>
        /// <param term='tag'>
        ///      Tag of the button to find.
        /// </param>
        private CommandBarButton GetButton(string tag)
        {
            return (CommandBarButton)GetCurrentToolbar().FindControl(
                        MsoControlType.msoControlButton,
                        Missing.Value,
                        tag,
                        true /*Visible*/,
                        false /*Recursive*/);
        }

        /// <summary>
        /// Called with the toggle button is clicked.
        /// </summary>
        private void OnToggleClick(CommandBarButton Ctrl, ref bool CancelDefault)
        {
            try
            {
                int hashCode = _applicationObject.ActiveWindow.GetHashCode();
                CustomTaskPane taskPane = (CustomTaskPane)_taskPanes[hashCode];

                if (Ctrl.Caption == ShowPaneCaption)
                {
                    if (taskPane == null)
                    {
                        taskPane = _ctpFactory.CreateCTP(
                                        "FormDataTaskPaneActiveX.FormDataTPAxCtl",
                                        "Form Data View",
                                        Missing.Value);

                        taskPane.VisibleStateChange += new _CustomTaskPaneEvents_VisibleStateChangeEventHandler(taskPane_VisibleStateChange);

                        _taskPanes.Add(hashCode, taskPane);
                    }

                    IFormDataTPActiveXCtl ctpActiveXCtrl = (IFormDataTPActiveXCtl)taskPane.ContentControl;

                    taskPane.Visible = true;
                    Ctrl.Caption = HidePaneCaption;

                    ctpActiveXCtrl.Application = taskPane.Application;
                }
                else
                {
                    CommandBarButton refreshButton = GetButton(RefreshButtonTag);
                    IFormDataTPActiveXCtl ctpActiveXCtrl = (IFormDataTPActiveXCtl)taskPane.ContentControl;

                    taskPane.Visible = false;
                    Ctrl.Caption = ShowPaneCaption;

                    refreshButton.Enabled = false;

                    ctpActiveXCtrl.Application = null;
                }
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, DialogCaption);
            }
        }

        private void taskPane_VisibleStateChange(CustomTaskPane CustomTaskPaneInst)
        {
            if (!CustomTaskPaneInst.Visible)
            {
                CommandBarButton toggleButton = GetButton(ToggleButtonTag);
                toggleButton.Caption = ShowPaneCaption;

                CommandBarButton refreshButton = GetButton(RefreshButtonTag);
                refreshButton.Enabled = false;
            }
        }

        /// <summary>
        /// Called with the toggle button is clicked.
        /// </summary>
        private void OnRefreshClick(CommandBarButton Ctrl, ref bool CancelDefault)
        {
            try
            {
                CustomTaskPane taskPane = (CustomTaskPane)_taskPanes[_applicationObject.ActiveWindow.GetHashCode()];
                IFormDataTPActiveXCtl ctpActiveXCtrl = (IFormDataTPActiveXCtl)taskPane.ContentControl;

                // Setting the Application property refreshes the XML in
                // the Custom Task Pane.
                ctpActiveXCtrl.Application = taskPane.Application;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, DialogCaption);
            }
        }

        #region ICustomTaskPaneConsumer Members

        public void CTPFactoryAvailable(ICTPFactory CTPFactoryInst)
        {
            try
            {
                _ctpFactory = CTPFactoryInst;
            }
            catch (Exception e)
            {
                MessageBox.Show(e.Message, DialogCaption);
            }
        }

        #endregion

        #region IFormDataViewAddIn Methods

        public void OnTaskPaneReady()
        {
            CommandBarButton refreshButton = GetButton(RefreshButtonTag);
            refreshButton.Enabled = true;
        }
        #endregion

        #region Data Members

        private ApplicationClass _applicationObject;
        private ApplicationEvents _applicationEvents;
        private ICTPFactory _ctpFactory;
        private ListDictionary _taskPanes;

        private const string DialogCaption = "Form Data View Add-in";
        private const string ToolbarName = "Form Data View Toolbar";
        private const string TaskPaneName = "Form Data View TaskPane";
        private const string ToggleButtonTag = "ToggleButton";
        private const string RefreshButtonTag = "RefreshButton";
        private const string ShowPaneCaption = "Show Form Data View Pane";
        private const string HidePaneCaption = "Hide Form Data View Pane";
        private const string RefreshCaption = "Refresh XML";

        #endregion
    }
}