// CFormDataTPActiveXCtl.cpp : Implementation of CFormDataTPActiveXCtl
#include "stdafx.h"
#include "FormDataTPActiveXCtl.h"
#include <msxml2.h>
#include <shlwapi.h>

static const CComBSTR k_bstrEmpty(L"");
static const CComBSTR k_bstrNewline(L"\n");

bool IsStringWhitespace(BSTR bstr);

// IFormDataTPActiveXCtl Methods
STDMETHODIMP CFormDataTPActiveXCtl::put_Application(IDispatch* pApplication)
{
    _spApplication = pApplication;

    if (_spApplication != NULL)
    {
        HRESULT hr;
        WindowPtr spWindow;

        hr = _spApplication->get_ActiveWindow(&spWindow);

        if (SUCCEEDED(hr))
        {
            _XDocumentPtr spXDocument;
            CComPtr<IXMLDOMDocument>    spDOM;

            hr = spWindow->get_XDocument(&spXDocument);

            if (SUCCEEDED(hr))
            {
                hr = spXDocument->get_DOM(&spDOM);
                if (SUCCEEDED(hr))
                {
                    _bstrXML.Empty();
                    FormatXml(spDOM, &_bstrXML, 0);

                    FireViewChange();
                    FireTaskPaneReady();
                }
            }
        }
    }

    return S_OK;
}

// Helper Methods
void CFormDataTPActiveXCtl::FireTaskPaneReady()
{
    if (_spApplication == NULL)
    {
        ATLASSERT(FALSE);
        return;
    }

    _Application3Ptr spApplication3;

    // Get the Add-in object from the COMAddIns collection and 
    // fire the OnTaskPaneReady event.
    HRESULT hr = _spApplication.QueryInterface(__uuidof(_Application3), &spApplication3);
    if (SUCCEEDED(hr))
    {
        CComDispatchDriver spdispCOMAddIns;

        hr = spApplication3->get_COMAddIns(&spdispCOMAddIns);
        if (SUCCEEDED(hr))
        {
            COMAddInsPtr spCOMAddIns;

            hr = spdispCOMAddIns->QueryInterface(__uuidof(COMAddIns), (void**)&spCOMAddIns);
            if (SUCCEEDED(hr))
            {
                COMAddInPtr spCOMAddIn;

                hr = spCOMAddIns->Item(&CComVariant(L"FormDataTaskPane.Connect"), &spCOMAddIn);
                if (SUCCEEDED(hr))
                {
                    CComDispatchDriver spdispCOMAddIn;

                    hr = spCOMAddIn->get_Object(&spdispCOMAddIn);
                    if (SUCCEEDED(hr) && spdispCOMAddIn != NULL)
                    {
                        IFormDataViewAddInPtr spFormDataViewAddIn;

                        hr = spdispCOMAddIn->QueryInterface(__uuidof(IFormDataViewAddIn), (void**)&spFormDataViewAddIn);
                        if (SUCCEEDED(hr))
                        {
                            spFormDataViewAddIn->OnTaskPaneReady();
                        }
                    }
                }
            }
        }
    }
}

// FormatXml: formats the XML for display in the task pane.
void CFormDataTPActiveXCtl::FormatXml(IXMLDOMNode* pNode, BSTR* pbstrXML, int level)
{
    CComBSTR bstrXML;

    if (pNode == NULL)
        return;

    bstrXML.Attach(*pbstrXML);

    VARIANT_BOOL vfHasChildren;
    HRESULT hr = pNode->hasChildNodes(&vfHasChildren);

    if (SUCCEEDED(hr) && vfHasChildren)
    {
        CComPtr<IXMLDOMNodeList> spChildren;
        
        hr = pNode->get_childNodes(&spChildren);

        if (SUCCEEDED(hr) && spChildren != NULL)
        {
            long cChildren = 0;

            spChildren->get_length(&cChildren);

            for (long i = 0; i < cChildren; i++)
            {
                CComPtr<IXMLDOMNode> spChild;

                hr = spChildren->get_item(i, &spChild);
                if (SUCCEEDED(hr) && spChild != NULL)
                {
                    CComBSTR bstrName;
                    DOMNodeType nodeType = NODE_INVALID;

                    spChild->get_nodeType(&nodeType);
                    if (nodeType != NODE_INVALID && nodeType != NODE_PROCESSING_INSTRUCTION)
                    {
                        if (nodeType == NODE_ELEMENT || nodeType == NODE_TEXT)
                        {
                            spChild->get_nodeName(&bstrName);
                        }

                        if (bstrName != k_bstrEmpty)
                        {
                            switch(nodeType)
                            {
                                case NODE_ELEMENT:
                                    for (int j = 0; j < level; j++)
                                    {
                                        bstrXML.Append(L"    ");
                                    }

                                    bstrXML.AppendBSTR(bstrName);

                                    // Output attributes
                                    OutputAttributes(spChild, bstrXML, level);
                                    bstrXML.Append(L"\n");
                                    break;

                                case NODE_TEXT:
                                {
                                    CComVariant varValue;

                                    spChild->get_nodeValue(&varValue);
                                    if (SUCCEEDED(varValue.ChangeType(VT_BSTR)))
                                    {
                                        CComBSTR bstrValue;

                                        bstrValue.Attach(V_BSTR(&varValue));

                                        if (!IsStringWhitespace(V_BSTR(&varValue)))
                                        {
                                            for (int j = 0; j < level; j++)
                                            {
                                                bstrXML.Append(L"    ");
                                            }

                                            bstrXML.Append(L": ");
                                            bstrXML.AppendBSTR(bstrValue);
                                            bstrXML.Append(L"\n");
                                        }
                                    }

                                    break;
                                }
                            }
                        }
                    }
                }

                FormatXml(spChild, &bstrXML, level+1);
            }
        }
    }

    *pbstrXML = bstrXML.Detach();
}

// Helper method for FormatXml to output the attributes 
void CFormDataTPActiveXCtl::OutputAttributes(IXMLDOMNode* pParent, CComBSTR& bstrXML, int level)
{
    HRESULT hr;
    CComPtr<IXMLDOMNamedNodeMap> spAttributes;

    hr = pParent->get_attributes(&spAttributes);

    if (SUCCEEDED(hr))
    {
        CComPtr<IXMLDOMNode> spAttribute;

        spAttributes->nextNode(&spAttribute);

        while(spAttribute != NULL)
        {
            CComBSTR bstrName;

            hr = spAttribute->get_nodeName(&bstrName);

            if (SUCCEEDED(hr)
                && (StrNCmp(bstrName, L"xml:", 4) != 0) 
                && (StrNCmp(bstrName, L"xsi:", 4) != 0) 
                && (StrNCmp(bstrName, L"xmlns:", 6) != 0))
            {
                CComBSTR bstrValue;

                bstrXML.Append("\n");

                for (int i = 0; i < level+1; i++)
                {
                    bstrXML.Append(L"    ");
                }

                bstrXML.Append(L"@");
                bstrXML.Append(bstrName);

                spAttribute->get_text(&bstrValue);

                if (bstrValue != k_bstrEmpty)
                {
                    bstrXML.Append(L": ");
                    bstrXML.AppendBSTR(bstrValue);
                }
            }

            spAttribute = NULL;
            spAttributes->nextNode(&spAttribute);
        }
    }
}

bool IsStringWhitespace(BSTR bstr)
{
    long cChars = SysStringLen(bstr);

    for (int i = 0; i < cChars; i++)
    {
        if (!IsCharSpace(bstr[i]))
            return false;
    }

    return true;
}