using System;
using System.Windows.Forms;
using Microsoft.VisualStudio.Tools.Applications.Runtime;
using Microsoft.Office.Interop.InfoPath.Xml;
using InfoPath = Microsoft.Office.Interop.InfoPath;
using Office = Microsoft.Office.Core;

namespace TrackBarTaskPane
{
    public partial class ThisAddIn
    {
        private void ThisAddIn_Startup(object sender, System.EventArgs e)
        {
            _application = (InfoPath.ApplicationClass)this.Application;

            _applicationEvents = (InfoPath.ApplicationEvents)_application.Events;

            _applicationEvents.XDocumentOpen += new InfoPath._ApplicationEvents_XDocumentOpenEventHandler(_applicationEvents_XDocumentOpen);
            _applicationEvents.NewXDocument += new InfoPath._ApplicationEvents_NewXDocumentEventHandler(_applicationEvents_NewXDocument);
            _applicationEvents.XDocumentBeforeClose += new InfoPath._ApplicationEvents_XDocumentBeforeCloseEventHandler(_applicationEvents_XDocumentBeforeClose);

            _userControl = new TrackBarUserControl();

            this.CustomTaskPanes.Add(_userControl, "TrackBar Task Pane");
        }

        private void ThisAddIn_Shutdown(object sender, System.EventArgs e)
        {
        }

        /// <summary>
        /// Called when an existing form is opened
        /// </summary>
        private void _applicationEvents_XDocumentOpen(InfoPath._XDocument pDocument)
        {
            OnFormOpened(pDocument);
        }

        /// <summary>
        /// Called when a new form is created from a form template
        /// </summary>
        private void _applicationEvents_NewXDocument(InfoPath._XDocument pDocument)
        {
            OnFormOpened(pDocument);
        }

        /// <summary>
        /// Called when the document is about to close.
        /// </summary>
        private void _applicationEvents_XDocumentBeforeClose(InfoPath._XDocument pDocument, ref bool pfCancel)
        {
            // This custom task pane only works with one InfoPath window.
            // To make it work with multiple InfoPath windows, use the same
            // trick demonstrated in the FormDataTaskPane sample and explained
            // in the chapter in the book that discusses this sample.
            this.CustomTaskPanes[0].Visible = false;
        }

        /// <summary>
        /// Loads a form template into the form control and 
        /// shows the custom task pane.
        /// </summary>
        private void OnFormOpened(InfoPath._XDocument pDocument)
        {
            if (pDocument.DOM != null)
            {
                IXMLDOMNode rootNode = pDocument.DOM.selectSingleNode("*[not(self::processing-instruction())]");
                if (rootNode != null)
                {
                    _node = rootNode.selectSingleNode("*");
                    if (_node != null)
                    {
                        _userControl.TrackBar.ValueChanged += new EventHandler(TrackBar_ValueChanged);
                        _userControl.TrackBar.Enabled = true;
                    }
                }
            }

            // This custom task pane only works with one InfoPath window.
            // To make it work with multiple InfoPath windows, use the same
            // trick demonstrated in the FormDataTaskPane sample and explained
            // in Chapter 20.
            this.CustomTaskPanes[0].Visible = true;
        }

        void TrackBar_ValueChanged(object sender, EventArgs e)
        {
            _node.text = _userControl.TrackBar.Value.ToString();
        }

        #region VSTO generated code

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(ThisAddIn_Startup);
            this.Shutdown += new System.EventHandler(ThisAddIn_Shutdown);
        }

        #endregion

        #region Data Members

        private IXMLDOMNode _node = null;
        private InfoPath.ApplicationClass _application;
        private TrackBarUserControl _userControl;
        private InfoPath.ApplicationEvents _applicationEvents;

        #endregion
    }
}
